@@ -40,16 +40,12 @@
 #include <IOKit/IORangeAllocator.h>
 #include <IOKit/IODeviceTreeSupport.h>
 #include <IOKit/IOPlatformExpert.h>
-#include <IOKit/IOMapper.h>
 #include <IOKit/IOLib.h>
 #include <IOKit/assert.h>
 
 #include "AppleMacRiscPCI.h"
 
-#define NO_FASTWRITE		1
-#define NO_NVIDIA_FASTWRITE	1
-
-#define ALLOC_AGP_RANGE		0
+#define ALLOC_AGP_RANGE	0
 
 #ifndef kIOAGPCommandValueKey
 #define kIOAGPCommandValueKey	"IOAGPCommandValue"
@@ -65,8 +61,6 @@ OSDefineMetaClassAndStructors(AppleMacRiscVCI, AppleMacRiscPCI)
 
 OSDefineMetaClassAndStructors(AppleMacRiscAGP, AppleMacRiscPCI)
 
-OSDefineMetaClassAndStructors(AppleMacRiscHT, IOPCIBridge)
-
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 bool AppleMacRiscPCI::start( IOService * provider )
@@ -79,18 +73,16 @@ bool AppleMacRiscPCI::start( IOService * provider )
     IORegistryEntry *		bridge;
     OSData *			busProp;
 
-    IORegistryEntry		*uniNRegEntry;
+    IORegistryEntry		*parent, *uniNRegEntry;
     OSData               	*tmpData;
     
-    // Match the name of the provider to weed out the memory controller
     if( !IODTMatchNubWithKeys(provider, "('pci', 'vci')"))
 	return( false);
 
-    // Match the compatible property to find modern MacRISC PCI bridges
-    if( IODTMatchNubWithKeys(provider, "('uni-north', 'u3-agp')"))
+    if( IODTMatchNubWithKeys(provider, "'uni-north'"))
 	configDataOffsetMask = 0x7;
     else
-	configDataOffsetMask = 0x3;
+	configDataOffsetMask = 0x3;      
 
     if( 0 == (lock = IOSimpleLockAlloc()))
 	return( false );
@@ -104,19 +96,18 @@ bool AppleMacRiscPCI::start( IOService * provider )
 
     //
     // Disable AGP for P58/P69 U2 (v2.0) due to stablity issues
+    //
+    parent = provider->getParentEntry(gIODTPlane);
+    
     // Get the Uni-N Version.
-    uniNRegEntry = IORegistryEntry::fromPath("/uni-n", gIODTPlane);
-    if (!uniNRegEntry)
-	uniNRegEntry = IORegistryEntry::fromPath("/u3", gIODTPlane);
-    if (uniNRegEntry)
+    uniNRegEntry = parent->fromPath("/uni-n", gIODTPlane);
+        
+    if (uniNRegEntry != NULL)
     {
         tmpData = OSDynamicCast(OSData, uniNRegEntry->getProperty("device-rev"));
-        if (tmpData)
-		{
+            
+        if (tmpData != NULL)
             uniNVersion = *(long *)tmpData->getBytesNoCopy();
-	uniNRegEntry->release();
-			//uniNVersion &= 0x3f;
-		}
     }
             
     bridge = provider;
@@ -161,27 +152,19 @@ bool AppleMacRiscPCI::start( IOService * provider )
 	return( false);
 
     busProp = (OSData *) bridge->getProperty("bus-range");
-    if( busProp) {
-	secondaryBus = *((UInt32 *) busProp->getBytesNoCopy());
-	subordinateBus = *((UInt32 *) busProp->getBytesNoCopy() + 1);
-    }
+    if( busProp)
+	primaryBus = *((UInt32 *) busProp->getBytesNoCopy());
     
     return( super::start( provider));
 }
 
 bool AppleMacRiscPCI::configure( IOService * provider )
 {
-    UInt32	modeSelects, addressSelects;
+    UInt32	addressSelects;
     UInt32	index;
     bool	ok;
 
-    if( getProvider()->getProperty( "DisableRDG") != 0 ) {
-      modeSelects = configRead32( getBridgeSpace(), kMacRISCPCIModeSelect );
-      modeSelects |= kMacRISCPCIModeSelectRDGBit;
-      configWrite32( getBridgeSpace(), kMacRISCPCIModeSelect, modeSelects );
-    }
-
-    addressSelects = configRead32( getBridgeSpace(), kMacRISCPCIAddressSelect );
+    addressSelects = configRead32( getBridgeSpace(), kMacRISCAddressSelect );
 
     coarseAddressMask	= addressSelects >> 16;
     fineAddressMask	= addressSelects & 0xffff;
@@ -236,12 +219,12 @@ bool AppleMacRiscVCI::configure( IOService * provider )
 
 UInt8 AppleMacRiscPCI::firstBusNum( void )
 {
-    return( secondaryBus );
+    return( primaryBus );
 }
 
 UInt8 AppleMacRiscPCI::lastBusNum( void )
 {
-    return( subordinateBus );
+    return( firstBusNum() );
 }
 
 IOPCIAddressSpace AppleMacRiscPCI::getBridgeSpace( void )
@@ -249,8 +232,8 @@ IOPCIAddressSpace AppleMacRiscPCI::getBridgeSpace( void )
     IOPCIAddressSpace	space;
 
     space.bits = 0;
-    space.s.busNum = secondaryBus;
-    space.s.deviceNum = kPCIBridgeSelfDevice;
+    space.s.busNum = primaryBus;
+    space.s.deviceNum = kBridgeSelfDevice;
 
     return( space );
 }
@@ -261,9 +244,9 @@ inline bool AppleMacRiscPCI::setConfigSpace( IOPCIAddressSpace space,
     UInt32	addrCycle;
 
     offset &= 0xfc;
-    if( space.s.busNum == secondaryBus) {
+    if( space.s.busNum == primaryBus) {
 
-	if( space.s.deviceNum < kPCIBridgeSelfDevice)
+	if( space.s.deviceNum < kBridgeSelfDevice)
 	    return( false);
 
         // primary config cycle
@@ -287,6 +270,7 @@ inline bool AppleMacRiscPCI::setConfigSpace( IOPCIAddressSpace space,
     return( true );
 }
 
+
 UInt32 AppleMacRiscPCI::configRead32( IOPCIAddressSpace space,
 					UInt8 offset )
 {
@@ -427,259 +411,6 @@ void AppleMacRiscPCI::configWrite8( IOPCIAddressSpace space,
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-bool AppleMacRiscHT::start( IOService * provider )
-{
-    IOPhysicalAddress		ioPhysAddrs[2];
-    IOPhysicalAddress		ioPhysLengths[2];
-    OSArray *			array;
-    IODeviceMemory::InitElement	rangeList[ 4 ];
-    IODeviceMemory *		md;
-    IORegistryEntry *		bridge;
-    OSData *			busProp;
-
-    bridge = provider;
-
-    md = provider->getDeviceMemoryWithIndex(0);
-    if (md == 0)
-	return 0;
-    ioPhysAddrs[0] = md->getPhysicalAddress();
-    ioPhysLengths[0] = md->getLength();
-
-    md = provider->getDeviceMemoryWithIndex(1);
-    if (md == 0)
-	return 0;
-    ioPhysAddrs[1] = md->getPhysicalAddress();
-    ioPhysLengths[1] = md->getLength();
-
-    /* define more explicit ranges */
-    
-    rangeList[0].start	= ioPhysAddrs[0];
-    rangeList[0].length = 0x01000000UL;
-    rangeList[1].start	= ioPhysAddrs[0] + 0x01000000;
-    rangeList[1].length	= 0x01000000UL;
-    rangeList[2].start	= ioPhysAddrs[0] + 0x02000000;
-    rangeList[2].length = 0x00400000UL;
-    rangeList[3].start	= ioPhysAddrs[1];
-    rangeList[3].length = ioPhysLengths[1];
-
-    IORangeAllocator * platformRanges;
-    platformRanges = IOService::getPlatform()->getPhysicalRangeAllocator();
-    assert( platformRanges );
-    platformRanges->allocateRange( ioPhysAddrs[0], ioPhysLengths[0] );
-
-    array = IODeviceMemory::arrayFromList( rangeList, 4 );
-    if( !array)
-	return( false);
-
-    provider->setDeviceMemory( array );
-    ioMemory = (IODeviceMemory *) array->getObject( 2 );
-    array->release();
-
-    /* map registers */
-
-    if( (configType0Map = provider->mapDeviceMemoryWithIndex( 0 )))
-        configType0 = (volatile UInt8 *) configType0Map->getVirtualAddress();
-    if( (configType1Map = provider->mapDeviceMemoryWithIndex( 1 )))
-        configType1 = (volatile UInt8 *) configType1Map->getVirtualAddress();
-    if( (configSelfMap = provider->mapDeviceMemoryWithIndex( 3 )))
-        configSelf = (volatile UInt8 *) configSelfMap->getVirtualAddress();
-
-    if( !configType0 || !configType1 || !configSelf)
-	return( false);
-
-    busProp = (OSData *) bridge->getProperty("bus-range");
-    if( busProp)
-	primaryBus = *((UInt32 *) busProp->getBytesNoCopy());
-
-    return( super::start( provider));
-}
-
-bool AppleMacRiscHT::configure( IOService * provider )
-{
-    UInt32	addressSelects;
-    UInt32	index;
-    bool	ok;
-
-    addressSelects = configRead32( getBridgeSpace(), kMacRISCHTAddressSelect );
-
-    coarseAddressMask	= addressSelects >> 16;
-    fineAddressMask	= addressSelects & 0xffff;
-
-    for( index = 0; index < 15; index++ ) {
-	if( coarseAddressMask & (1 << index)) {
-	    ok = addBridgeMemoryRange( index << 28, 0x10000000, true );
-	}
-    }
-
-//    if( coarseAddressMask & (1 << 15))	// F segment
-	for( index = 0; index < 15; index++ ) {
-	    if( fineAddressMask & (1 << index)) {
-		ok = addBridgeMemoryRange( (0xf0 | index) << 24,
-						0x01000000, true );
-	    }
-	}
-
-    ok = addBridgeIORange( 0, 0x10000 );
-
-    return( super::configure( provider));
-}
-
-void AppleMacRiscHT::free()
-{
-    if( configType0Map)
-	configType0Map->release();
-    if( configType1Map)
-	configType1Map->release();
-    if( configSelfMap)
-	configSelfMap->release();
-
-    super::free();
-}
-
-IODeviceMemory * AppleMacRiscHT::ioDeviceMemory( void )
-{
-    return( ioMemory);
-}
-
-IOPCIAddressSpace AppleMacRiscHT::getBridgeSpace( void )
-{
-    IOPCIAddressSpace	space;
-
-    space.bits = 0;
-    space.s.busNum = primaryBus;
-    space.s.deviceNum = kHTBridgeSelfDevice;
-
-    return( space );
-}
-
-volatile UInt8 * AppleMacRiscHT::setConfigSpace( IOPCIAddressSpace space,
-                                            UInt8 offset )
-{
-    volatile UInt8 * configData;
-
-    offset &= 0xfc;
-    if( space.s.busNum == primaryBus) {
-
-	if( space.s.deviceNum == kHTBridgeSelfDevice) {
-            // primary config cycle on self
-            configData = configSelf + (offset << 2);
-        } else {
-            // primary config cycle
-            configData = configType0 + ((space.bits & 0xffffff00) | offset);
-        }
-    } else {
-
-        // pass thru config cycle
-        configData = configType1 + ((space.bits & 0xffffff00) | offset);
-    }
-
-    return( configData );
-}
-
-UInt32 AppleMacRiscHT::configRead32( IOPCIAddressSpace space,
-					UInt8 offset )
-{
-    volatile UInt8 * configData;
-    UInt32	     data;
-
-    configData = setConfigSpace( space, offset );
-
-    offset = offset & 3 & 4;
-
-    data = OSReadSwapInt32( configData, offset );
-    eieio();
-
-    return( data );
-}
-
-void AppleMacRiscHT::configWrite32( IOPCIAddressSpace space, 
-					UInt8 offset, UInt32 data )
-{
-    volatile UInt8 * configData;
-
-    configData = setConfigSpace( space, offset );
-
-    offset = offset & 3 & 4;
-
-    OSWriteSwapInt32( configData, offset, data );
-    eieio();
-    /* read to sync */
-    (void) OSReadSwapInt32( configData, offset );
-    eieio();
-    sync();
-    isync();
-}
-
-UInt16 AppleMacRiscHT::configRead16( IOPCIAddressSpace space,
-					UInt8 offset )
-{
-    volatile UInt8 * configData;
-    UInt16	     data;
-
-    configData = setConfigSpace( space, offset );
-
-    offset = offset & 3 & 6;
-
-    data = OSReadSwapInt16( configData, offset );
-    eieio();
-
-    return( data );
-}
-
-void AppleMacRiscHT::configWrite16( IOPCIAddressSpace space, 
-					UInt8 offset, UInt16 data )
-{
-    volatile UInt8 * configData;
-
-    configData = setConfigSpace( space, offset );
-
-    offset = offset & 3 & 6;
-
-    OSWriteSwapInt16( configData, offset, data );
-    eieio();
-    /* read to sync */
-    (void) OSReadSwapInt16( configData, offset );
-    eieio();
-    sync();
-    isync();
-}
-
-UInt8 AppleMacRiscHT::configRead8( IOPCIAddressSpace space,
-					UInt8 offset )
-{
-    volatile UInt8 * configData;
-    UInt16	     data;
-
-    configData = setConfigSpace( space, offset );
-
-    offset = offset & 3;
-
-    data = configData[ offset ];
-    eieio();
-
-    return( data );
-}
-
-void AppleMacRiscHT::configWrite8( IOPCIAddressSpace space, 
-					UInt8 offset, UInt8 data )
-{
-    volatile UInt8 * configData;
-
-    configData = setConfigSpace( space, offset );
-
-    offset = offset & 3;
-
-    configData[ offset ] = data;
-    eieio();
-    /* read to sync */
-    data = configData[ offset ];
-    eieio();
-    sync();
-    isync();
-}
-
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-
 #undef super
 #define super AppleMacRiscPCI
 
@@ -690,28 +421,6 @@ bool AppleMacRiscAGP::configure( IOService * provider )
     if( !findPCICapability( getBridgeSpace(), kIOPCIAGPCapability, &targetAGPRegisters ))
 	return( false );
 
-    if( false && (uniNVersion >= 0x10)) {
-	// causes problems with nv25 SBA
-	gartCtrl |= kGART_DIS_SBA_DET;
-    }
-
-    isU3 = IODTMatchNubWithKeys(provider, "u3-agp");
-    if (isU3) {
-	
-        uniNVersion &= 0x3f;
-
-        gartCtrl |= kGART_PERF_RD;
-	
-        if(uniNVersion > 0x33)
-        {
-            /* B2B_GNT seems broken on U3 Twins */
-            //gartCtrl |= kGART_B2B_GNT;
-            //gartCtrl |= kGART_FAST_DDR;
-        }
-	
-        isU32     = (uniNVersion > 0x30);
-    }
-
     return( super::configure( provider));
 }
 
@@ -728,53 +437,18 @@ IOPCIDevice * AppleMacRiscAGP::createNub( OSDictionary * from )
 	    && findPCICapability( space, kIOPCIAGPCapability, &masterAGPRegisters ));
 
     // more of P58/P69 AGP disable code from start
-    if( isAGP 
-     && ((0 != getProvider()->getProperty("AAPL,noAGP")) || (uniNVersion == 0x20)))
+    if (uniNVersion == 0x20)
     {
         isAGP = false;
-        IOLog("AGP disabled on this machine\n");
+        IOLog("AGP mode disabled on this machine\n");
     }
     
-    if( isAGP)
-    {
-	UInt64	   flags;
-	OSNumber * num;
-
-	num = OSDynamicCast(OSNumber, getProperty(kIOAGPBusFlagsKey));
-	if (num)
-	    flags = num->unsigned64BitValue();
-	else
-	    flags = 0;
-
-	if (isU32)
-	    flags &= ~kIOAGPGartInvalidate;
-	else
-	    flags |= kIOAGPGartInvalidate;
-#if 0
-	if (isU3 || (uniNVersion < 0x08))
-	    flags |= kIOAGPDisablePageSpans;
-#else
-	if ((uniNVersion < 0x08) || (isU3 && (uniNVersion < 0x33)))
-	    flags |= kIOAGPDisablePageSpans;
-#endif
-	if (isU3 && (uniNVersion < 0x34))
-	    flags |= kIOAGPDisableUnaligned;
-	if (uniNVersion < 0x20)
-	    flags |= kIOAGPDisableAGPWrites;
-	if (uniNVersion == 0x30)
-	    flags |= kIOAGPDisablePCIReads | kIOAGPDisablePCIWrites;
-
-	num = OSNumber::withNumber(flags, 64);
+    if( isAGP) {
 	nub = new IOAGPDevice;
-        if (nub)
+        if( nub)
             ((IOAGPDevice *)nub)->masterAGPRegisters = masterAGPRegisters;
-	if (num)
-	{
-	    from->setObject(kIOAGPBusFlagsKey, num);
-	    num->release();
-	}
-    }
-    else
+	from->setObject( kIOAGPBusFlagsKey, getProperty(kIOAGPBusFlagsKey));
+    } else
         nub = super::createNub( from );
 
     return( nub );
@@ -790,8 +464,7 @@ IOReturn AppleMacRiscAGP::createAGPSpace( IOAGPDevice * master,
     IOReturn		err;
     IOPCIAddressSpace 	target = getBridgeSpace();
     IOPhysicalLength	agpLength;
-    UInt64		gartPhys;
-    OSData *		data;
+    IOPhysicalAddress	gartPhys;
 
     enum { agpSpacePerPage = 4 * 1024 * 1024 };
     enum { agpBytesPerGartByte = 1024 };
@@ -799,62 +472,6 @@ IOReturn AppleMacRiscAGP::createAGPSpace( IOAGPDevice * master,
 
     destroyAGPSpace( master );
 
-    if (isU32 && !dummyPage)
-	dummyPage = IOBufferMemoryDescriptor::withOptions(0, page_size, page_size);
-    if (dummyPage) {
-	IOPhysicalLength len;
-	dummyPhys = dummyPage->getPhysicalSegment64(0, &len);
-        configWrite32( target, kUniNDUMMY_PAGE, dummyPhys >> PAGE_SHIFT );
-    }
-
-    agpCommandMask = 0xffffffff;
-//  agpCommandMask &= ~kIOAGPSideBandAddresssing;
-
-    if (isU3)
-	agpCommandMask &= ~kIOAGPFastWrite;
-
-    {
-	// There's an nVidia NV11 ROM (revision 1017) that says that it can do fast writes,
-	// but can't, and can often lock the machine up when fast writes are enabled.
-	#define kNVIDIANV11EntryName	"NVDA,NVMac"
-	#define kNVIDIAEntryName	"NVDA,"
-	#define kNVROMRevPropertyName 	"rom-revision"
-	#define kNVBadRevision		'1017'
-
-#if NO_NVIDIA_FASTWRITE
-
-	if( 0 == strncmp( kNVIDIAEntryName, master->getName(), strlen(kNVIDIAEntryName)))
-        {
-	    agpCommandMask &= ~kIOAGPFastWrite;			// NV34 systems (Q26B/Q54) has issues with this
-	    if (!isU3)
-		agpCommandMask &= ~kIOAGPSideBandAddresssing;	// NV34 systems (Q26B/Q54) has issues with this
-        }
-
-#else	/* NO_NVIDIA_FASTWRITE */
-
-	const UInt32    badRev = kNVBadRevision;
-
-	if( (0 == strcmp( kNVIDIANV11EntryName, master->getName()))
-	 && (data = OSDynamicCast(OSData, master->getProperty(kNVROMRevPropertyName)))
-	 && (data->isEqualTo( &badRev, sizeof(badRev)))	 )
-	    agpCommandMask &= ~kIOAGPFastWrite;
-
-#endif	/* !NO_NVIDIA_FASTWRITE */
-    }
-
-#if NO_FASTWRITE
-    agpCommandMask &= ~kIOAGPFastWrite;
-#endif	/* !NO_FASTWRITE */
-
-    if ((data = OSDynamicCast(OSData, getProvider()->getProperty("AAPL,agp-clear"))))
-	agpCommandMask &= ~*((UInt32 *)data->getBytesNoCopy());
-    if ((data = OSDynamicCast(OSData, getProvider()->getProperty("AAPL,agp-set"))))
-	agpCommandSet |= *((UInt32 *)data->getBytesNoCopy());
-    if ((data = OSDynamicCast(OSData, master->getProperty("AAPL,agp-clear"))))
-	agpCommandMask &= ~*((UInt32 *)data->getBytesNoCopy());
-    if ((data = OSDynamicCast(OSData, master->getProperty("AAPL,agp-set"))))
-	agpCommandSet |= *((UInt32 *)data->getBytesNoCopy());
-
     agpLength = *length;
     if( !agpLength)
 	agpLength = 32 * 1024 * 1024;
@@ -863,17 +480,14 @@ IOReturn AppleMacRiscAGP::createAGPSpace( IOAGPDevice * master,
 
     err = kIOReturnVMError;
     do {
-        ppnum_t physPage;
 
 	gartLength = agpLength / agpBytesPerGartByte;
-        gartHandle = IOMapper::NewARTTable(gartLength, (void **) &gartArray, &physPage);
-        if (!gartHandle)
-            continue;
-        gartPhys = ptoa_64(physPage);
-
+	gartArray = (volatile UInt32 *) IOMallocContiguous( 
+				gartLength, 4096, &gartPhys );
+	if( !gartArray)
+	    continue;
         // IOMapPages( kernel_map, gartArray, gartPhys, gartLength, kIOMapInhibitCache );
-	bzero( (void *) gartArray, gartLength);
-	flush_dcache( (vm_offset_t) gartArray, gartLength, false);
+        bzero( (void *) gartArray, gartLength);
 
 #if ALLOC_AGP_RANGE
         IORangeAllocator * platformRanges
@@ -904,15 +518,11 @@ IOReturn AppleMacRiscAGP::createAGPSpace( IOAGPDevice * master,
         configWrite32( target, kMacRISCAddressSelect,
 				(coarseAddressMask << 16) | fineAddressMask );
 #endif
+        configWrite32( target, kUniNAGP_BASE, agpBaseIndex << 28 );
 
-	UInt32 gartBaseExt = 0;
-	if (isU3)
-	    gartBaseExt = (gartPhys >> 32);
-
-        configWrite32( target, kUniNAGP_BASE, (agpBaseIndex << 28) | gartBaseExt );
-
+        assert( 0 == (gartPhys & 0xfff));
         configWrite32( target, kUniNGART_BASE,
-     			(gartPhys & ~PAGE_MASK) | (agpLength / agpSpacePerPage));
+     			gartPhys | (agpLength / agpSpacePerPage));
 
         err = kIOReturnSuccess;
 
@@ -975,104 +585,52 @@ IORangeAllocator * AppleMacRiscAGP::getAGPRangeAllocator(
 }
 
 IOOptionBits AppleMacRiscAGP::getAGPStatus( IOAGPDevice * master,
-					    IOOptionBits options )
+					    IOOptionBits options = 0 )
 {
     IOPCIAddressSpace 	target = getBridgeSpace();
 
     return( configRead32( target, kUniNINTERNAL_STATUS ) );
 }
 
-inline void AppleMacRiscAGP::configSetClearMask( IOPCIAddressSpace space, 
-						 UInt8 offset, UInt32 data, UInt32 mask )
-{
-    IOInterruptState ints;
-
-    ints = IOSimpleLockLockDisableInterrupt( lock );
-
-    if( setConfigSpace( space, offset )) {
-
-        offset = offset & configDataOffsetMask & 4;
-
-        OSWriteSwapInt32( configData, offset, data | mask );
-        eieio();
-	/* read to sync */
-        (void) OSReadSwapInt32( configData, offset );
-        eieio();
-	sync();
-	isync();
-
-        OSWriteSwapInt32( configData, offset, data );
-        eieio();
-	/* read to sync */
-        (void) OSReadSwapInt32( configData, offset );
-        eieio();
-	sync();
-	isync();
-    }
-
-    IOSimpleLockUnlockEnableInterrupt( lock, ints );
-}
-
 IOReturn AppleMacRiscAGP::commitAGPMemory( IOAGPDevice * master, 
 				      IOMemoryDescriptor * memory,
 				      IOByteCount agpOffset,
-				      IOOptionBits options )
+				      IOOptionBits options = 0 )
 {
     IOPCIAddressSpace 	target = getBridgeSpace();
     IOReturn		err = kIOReturnSuccess;
     UInt32		offset = 0;
-    addr64_t		phys64;
+    IOPhysicalAddress	physAddr;
     IOByteCount		len;
-    IOByteCount         agpFlushStart, flushLength;
 
 //    ok = agpRange->allocate( memory->getLength(), &agpOffset );
 
     assert( agpOffset < systemLength );
     agpOffset /= (page_size / 4);
-    agpFlushStart = agpOffset;
+    while( (physAddr = memory->getPhysicalSegment( offset, &len ))) {
 
-    while( (phys64 = memory->getPhysicalSegment64(offset, &len)))
-    {
 	offset += len;
-	len = (len + PAGE_MASK) & ~PAGE_MASK;
+	len = (len + 0xfff) & ~0xfff;
 	while( len > 0) {
-	    if (isU3)
-		OSWriteBigInt32( gartArray, agpOffset,
-				    (((UInt32)(phys64 >> PAGE_SHIFT)) | 0x80000000));
-	    else
-		OSWriteLittleInt32( gartArray, agpOffset,
-				    ((phys64 & ~PAGE_MASK) | 1));
+	    OSWriteLittleInt32( gartArray, agpOffset,
+				((physAddr & ~0xfff) | 1));
 	    agpOffset += 4;
-	    phys64 += page_size;
+	    physAddr += page_size;
 	    len -= page_size;
 	}
     }
-
-    // Just to be paranoid, I want to feed nice happy values to flush_dcache.
-    // I want the start address to be cache line aligned, and I want the size
-    // to be a multiple of cache line size.  For now I'm going to assume that
-    // any CPU hooked to an AGP chipset using this driver has a 32-byte cache
-    // line size.  Ugh.
-    agpFlushStart &= ~31; // Round down to cache line boundary
-    flushLength = (agpOffset - agpFlushStart);
-    // Round up to 32 bytes, which hopefully is safe since we pushed the start
-    // down to a 32-byte boundary first.
-    flushLength = (flushLength + 31) & ~31;
-    sync();
-    isync();
-    flush_dcache( ((vm_offset_t) gartArray)+agpFlushStart, flushLength, false);
-    sync();
-    isync();
+#if 1
+    flush_dcache( (vm_offset_t) gartArray, gartLength, false);
     len = OSReadLittleInt32( gartArray, agpOffset - 4 );
     sync();
     isync();
+#endif
 
-    if (isU32) {
-	configSetClearMask( target, kUniNGART_CTRL, gartCtrl | kGART_EN, kGART_INV );
-    
-    } else if( kIOAGPGartInvalidate & options) {
-	configSetClearMask( target, kUniNGART_CTRL, gartCtrl | kGART_EN, kGART_INV );
-	configSetClearMask( target, kUniNGART_CTRL, gartCtrl | kGART_EN, kGART_2xRESET );
+    if( kIOAGPGartInvalidate & options) {
+        configWrite32( target, kUniNGART_CTRL, kGART_EN | kGART_INV );
+        configWrite32( target, kUniNGART_CTRL, kGART_EN );
+        configWrite32( target, kUniNGART_CTRL, kGART_EN | kGART_2xRESET);
+        configWrite32( target, kUniNGART_CTRL, kGART_EN );
     }
 
     return( err );
@@ -1086,7 +644,6 @@ IOReturn AppleMacRiscAGP::releaseAGPMemory( IOAGPDevice * master,
     IOPCIAddressSpace 	target = getBridgeSpace();
     IOReturn		err = kIOReturnSuccess;
     IOByteCount		length;
-    IOByteCount         agpFlushStart, flushLength;
 
     if( !memory)
 	return( kIOReturnBadArgument );
@@ -1098,36 +655,24 @@ IOReturn AppleMacRiscAGP::releaseAGPMemory( IOAGPDevice * master,
 
 //    agpRange->deallocate( agpOffset, length );
 
-    length = (length + PAGE_MASK) & ~PAGE_MASK;
+    length = (length + 0xfff) & ~0xfff;
     agpOffset /= page_size;
-    agpFlushStart = agpOffset;
     while( length > 0) {
-	gartArray[agpOffset++] = 0;
+	gartArray[ agpOffset++ ] = 0;
 	length -= page_size;
     }
-    
-    agpFlushStart *= 4;
-    agpOffset *= 4;
-    agpFlushStart &= ~31;
-    flushLength = (agpOffset - agpFlushStart);  // Flush full cache lines
-    // Round up to 32 bytes, which hopefully is safe since we pushed the start
-    // down to a 32-byte boundary first.
-    flushLength = (flushLength + 31) & ~31;
-    sync();
-    isync();
-    flush_dcache( ((vm_offset_t) gartArray)+agpFlushStart, flushLength, false);
-    sync();
-    isync();
-    length = OSReadLittleInt32( gartArray, agpOffset - 4 );
+#if 1
+    flush_dcache( (vm_offset_t) gartArray, gartLength, false);
+    length = OSReadLittleInt32( gartArray, 4 * (agpOffset - 1) );
     sync();
     isync();
+#endif
 
-    if (isU32) {
-	configSetClearMask( target, kUniNGART_CTRL, gartCtrl | kGART_EN, kGART_INV );
-    
-    } else if( kIOAGPGartInvalidate & options) {
-	configSetClearMask( target, kUniNGART_CTRL, gartCtrl | kGART_EN, kGART_INV );
-	configSetClearMask( target, kUniNGART_CTRL, gartCtrl | kGART_EN, kGART_2xRESET );
+    if( kIOAGPGartInvalidate & options) {
+        configWrite32( target, kUniNGART_CTRL, kGART_EN | kGART_INV );
+        configWrite32( target, kUniNGART_CTRL, kGART_EN );
+        configWrite32( target, kUniNGART_CTRL, kGART_EN | kGART_2xRESET);
+        configWrite32( target, kUniNGART_CTRL, kGART_EN );
     }
 
     return( err );
@@ -1145,8 +690,6 @@ IOReturn AppleMacRiscAGP::setAGPEnable( IOAGPDevice * _master,
 
     if( enable) {
 
-        configWrite32( target, kUniNGART_CTRL, gartCtrl | 0);
-
 	targetStatus = configRead32( target,
                                      targetAGPRegisters + kIOPCIConfigAGPStatusOffset );
 	masterStatus = configRead32( master,
@@ -1154,7 +697,6 @@ IOReturn AppleMacRiscAGP::setAGPEnable( IOAGPDevice * _master,
 
 	command = kIOAGPSideBandAddresssing
                 | kIOAGPFastWrite
-                | kIOAGP3Mode
 		| kIOAGP4xDataRate | kIOAGP2xDataRate | kIOAGP1xDataRate;
 	command &= targetStatus;
 	command &= masterStatus;
@@ -1165,36 +707,15 @@ IOReturn AppleMacRiscAGP::setAGPEnable( IOAGPDevice * _master,
             IOLog("AGP 4x mode disabled on this machine\n");
         }
 
-        command &= agpCommandMask;
-        command |= agpCommandSet;
-
-	if (isU32)
-	    _master->setProperty("AAPL,agp3-mode", (command & kIOAGP3Mode) ? kOSBooleanTrue : kOSBooleanFalse);
-
-	if (isU32 && (command & kIOAGP3Mode))
-	{
-	    command &= ~kIOAGP4xDataRate;
-	    if( command & kIOAGP8xDataRateMode3)
-		command &= ~(kIOAGP4xDataRateMode3 | kIOAGPFastWrite);
-	    else if( 0 == (command & kIOAGP4xDataRateMode3))
-		return( kIOReturnUnsupported );
-	}
-	else
-	{
-	    if( command & kIOAGP4xDataRate)
-		command &= ~(kIOAGP2xDataRate | kIOAGP1xDataRate);
-	    else { 
-		if (isU3)
-		    return( kIOReturnUnsupported );
-    
-		if( command & kIOAGP2xDataRate)
-		    command &= ~(kIOAGP1xDataRate);
-		else if( 0 == (command & kIOAGP1xDataRate))
-		    return( kIOReturnUnsupported );
-	    }
-	}
+	if( command & kIOAGP4xDataRate)
+	    command &= ~(kIOAGP2xDataRate | kIOAGP1xDataRate);
+	else if( command & kIOAGP2xDataRate)
+	    command &= ~(kIOAGP1xDataRate);
+	else if( 0 == (command & kIOAGP1xDataRate))
+            return( kIOReturnUnsupported );
 
 	command |= kIOAGPEnable;
+        command &= ~kIOAGPFastWrite;
 
 	if( targetStatus > masterStatus)
 	    targetStatus = masterStatus;
@@ -1202,10 +723,12 @@ IOReturn AppleMacRiscAGP::setAGPEnable( IOAGPDevice * _master,
 
         _master->setProperty(kIOAGPCommandValueKey, &command, sizeof(command));
 
-        configWrite32( target, kUniNGART_CTRL, gartCtrl | kGART_EN );
-        configWrite32( target, kUniNGART_CTRL, gartCtrl | kGART_EN | kGART_INV );
-        configWrite32( target, kUniNGART_CTRL, gartCtrl | kGART_EN );
-
+#if 1
+        configWrite32( target, kUniNGART_CTRL, kGART_EN | kGART_INV );
+        configWrite32( target, kUniNGART_CTRL, kGART_EN );
+        configWrite32( target, kUniNGART_CTRL, kGART_EN | kGART_2xRESET);
+        configWrite32( target, kUniNGART_CTRL, kGART_EN );
+#endif
 	do {
 	    configWrite32( target, targetAGPRegisters + kIOPCIConfigAGPCommandOffset, command );
 	} while( (command & kIOAGPEnable) != 
@@ -1219,10 +742,10 @@ IOReturn AppleMacRiscAGP::setAGPEnable( IOAGPDevice * _master,
                             masterAGPRegisters + kIOPCIConfigAGPCommandOffset)));
 
 #if 0
-        configWrite32( target, kUniNGART_CTRL, gartCtrl | kGART_EN | kGART_INV );
-        configWrite32( target, kUniNGART_CTRL, gartCtrl | kGART_EN );
-        configWrite32( target, kUniNGART_CTRL, gartCtrl | kGART_EN | kGART_2xRESET);
-        configWrite32( target, kUniNGART_CTRL, gartCtrl | kGART_EN );
+        configWrite32( target, kUniNGART_CTRL, kGART_EN | kGART_INV );
+        configWrite32( target, kUniNGART_CTRL, kGART_EN );
+        configWrite32( target, kUniNGART_CTRL, kGART_EN | kGART_2xRESET);
+        configWrite32( target, kUniNGART_CTRL, kGART_EN );
 #endif
 
         _master->masterState |= kIOAGPStateEnabled;
@@ -1231,18 +754,16 @@ IOReturn AppleMacRiscAGP::setAGPEnable( IOAGPDevice * _master,
 
 	while( 0 == (kIOAGPIdle & configRead32( getBridgeSpace(),
 					kUniNINTERNAL_STATUS )))
-		{}
+	    {}
 
         configWrite32( master, masterAGPRegisters + kIOPCIConfigAGPCommandOffset, 0 );
         configWrite32( target, targetAGPRegisters + kIOPCIConfigAGPCommandOffset, 0 );
 #if 0
-        configWrite32( target, kUniNGART_CTRL, gartCtrl | kGART_EN | kGART_INV );
-        configWrite32( target, kUniNGART_CTRL, gartCtrl | 0 );
-        configWrite32( target, kUniNGART_CTRL, gartCtrl | kGART_2xRESET);
-        configWrite32( target, kUniNGART_CTRL, gartCtrl | 0 );
+        configWrite32( target, kUniNGART_CTRL, kGART_EN | kGART_INV );
+        configWrite32( target, kUniNGART_CTRL, 0 );
+        configWrite32( target, kUniNGART_CTRL, kGART_2xRESET);
+        configWrite32( target, kUniNGART_CTRL, 0 );
 #endif
-        configWrite32( target, kUniNGART_CTRL, gartCtrl | kGART_2xRESET );
-
         _master->masterState &= ~kIOAGPStateEnabled;
     }
 
@@ -1250,7 +771,7 @@ IOReturn AppleMacRiscAGP::setAGPEnable( IOAGPDevice * _master,
 }
 
 IOReturn AppleMacRiscAGP::resetAGPDevice( IOAGPDevice * master,
-                                          IOOptionBits options )
+                                          IOOptionBits options = 0 )
 {
     IOReturn ret;
 
@@ -1265,11 +786,12 @@ IOReturn AppleMacRiscAGP::resetAGPDevice( IOAGPDevice * master,
 #endif
         ret = kIOReturnSuccess;
     }
+
     return( ret );
 }
 
 IOReturn AppleMacRiscAGP::saveDeviceState( IOPCIDevice * device,
-                                           IOOptionBits options )
+                                           IOOptionBits options = 0 )
 {
     IOReturn		ret;
     IOAGPDevice *	agpDev;
@@ -1287,16 +809,16 @@ IOReturn AppleMacRiscAGP::saveDeviceState( IOPCIDevice * device,
     ret = super::saveDeviceState( device, options);
 
     if( agpDev && (ret == kIOReturnSuccess)) {
-        agpDev->savedConfig[ 16 ] = agpSave[0];
-        agpDev->savedConfig[ 17 ] = agpSave[1];
-        agpDev->savedConfig[ 18 ] = agpSave[2];
+        agpDev->savedConfig[ kUniNAGP_BASE / 4 ] = agpSave[0];
+        agpDev->savedConfig[ kUniNGART_BASE / 4 ] = agpSave[1];
+        agpDev->savedConfig[ (targetAGPRegisters + kIOPCIConfigAGPCommandOffset) / 4 ] = agpSave[2];
     }
 
     return( ret );
 }
 
 IOReturn AppleMacRiscAGP::restoreDeviceState( IOPCIDevice * device,
-                                              IOOptionBits options )
+                                              IOOptionBits options = 0 )
 {
     IOReturn		ret;
     IOAGPDevice *	agpDev;
@@ -1305,9 +827,9 @@ IOReturn AppleMacRiscAGP::restoreDeviceState( IOPCIDevice * device,
 
     agpDev = OSDynamicCast( IOAGPDevice, device);
     if( agpDev && device->savedConfig) {
-        agpSave[0] = agpDev->savedConfig[ 16 ];
-        agpSave[1] = agpDev->savedConfig[ 17 ];
-        agpSave[2] = agpDev->savedConfig[ 18 ];
+        agpSave[0] = agpDev->savedConfig[ kUniNAGP_BASE / 4 ];
+        agpSave[1] = agpDev->savedConfig[ kUniNGART_BASE / 4 ];
+        agpSave[2] = agpDev->savedConfig[ (targetAGPRegisters + kIOPCIConfigAGPCommandOffset) / 4 ];
     }
     
     ret = super::restoreDeviceState( device, options);
