@@ -30,22 +30,16 @@
 #ifndef _IOKIT_APPLEMACRISCPCI_H
 #define _IOKIT_APPLEMACRISCPCI_H
 
-#include <IOKit/IOBufferMemoryDescriptor.h>
 #include <IOKit/pci/IOPCIBridge.h>
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 enum {
-    kHTBridgeSelfDevice = 0,
-    kPCIBridgeSelfDevice = 11
+    kBridgeSelfDevice = 11
 };
 
 enum {
-  kMacRISCHTAddressSelect 	= 0x80,
-  kMacRISCPCIAddressSelect 	= 0x48,
-  kMacRISCPCIModeSelect		= 0x50,
-  kMacRISCPCIModeSelectRDGBit 	= 0x00080000,
-  kMacRISCPCIModeSelectWCBit    = 0x00100000
+    kMacRISCAddressSelect 	= 0x48
 };
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -65,10 +59,9 @@ protected:
 
     UInt16			coarseAddressMask;
     UInt16			fineAddressMask;
-    UInt8			secondaryBus;
-    UInt8			subordinateBus;
+    UInt8			primaryBus;
     UInt8			configDataOffsetMask;
-    UInt32			uniNVersion;
+    UInt32               	uniNVersion;
 
     inline bool setConfigSpace( IOPCIAddressSpace space, UInt8 offset );
     virtual UInt8 firstBusNum( void );
@@ -106,47 +99,6 @@ public:
 
 };
 
-class AppleMacRiscHT : public IOPCIBridge
-{
-    OSDeclareDefaultStructors(AppleMacRiscHT)
-
-protected:
-    IOMemoryMap *		configType0Map;
-    IOMemoryMap *		configType1Map;
-    IOMemoryMap *		configSelfMap;
-    IODeviceMemory *		ioMemory;
-
-    volatile UInt8	*	configType0;
-    volatile UInt8	*	configType1;
-    volatile UInt8	*	configSelf;
-
-    UInt16			coarseAddressMask;
-    UInt16			fineAddressMask;
-    UInt8			primaryBus;
-
-    inline volatile UInt8 * setConfigSpace( IOPCIAddressSpace space, UInt8 offset );
-
-public:
-    virtual bool start(	IOService * provider );
-    virtual bool configure( IOService * provider );
-
-    virtual void free();
-
-    virtual IODeviceMemory * ioDeviceMemory( void );
-
-    virtual UInt32 configRead32( IOPCIAddressSpace space, UInt8 offset );
-    virtual void configWrite32( IOPCIAddressSpace space,
-					UInt8 offset, UInt32 data );
-    virtual UInt16 configRead16( IOPCIAddressSpace space, UInt8 offset );
-    virtual void configWrite16( IOPCIAddressSpace space,
-					UInt8 offset, UInt16 data );
-    virtual UInt8 configRead8( IOPCIAddressSpace space, UInt8 offset );
-    virtual void configWrite8( IOPCIAddressSpace space,
-					UInt8 offset, UInt8 data );
-
-    virtual IOPCIAddressSpace getBridgeSpace( void );
-};
-
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 /* Definitions of UniNorth Target config registers */
@@ -154,25 +106,12 @@ enum {
     kUniNGART_BASE		= 0x8c,
     kUniNAGP_BASE		= 0x90,
     kUniNGART_CTRL		= 0x94,
-    kUniNINTERNAL_STATUS	= 0x98,
-    kUniNDUMMY_PAGE		= 0xa4
+    kUniNINTERNAL_STATUS	= 0x98
 };
 enum {
     kGART_INV			= 0x00000001,
     kGART_EN			= 0x00000100,
-    kGART_2xRESET		= 0x00010000,
-    kGART_DIS_SBA_DET		= 0x00020000,
-    kGART_SYNC_MODE		= 0x00040000,
-    kGART_PERF_RD		= 0x00080000,
-    kGART_B2B_GNT		= 0x00200000,
-    kGART_FAST_DDR		= 0x00400000
-};
-
-
-enum {
-    kIOAGP3Mode			= 0x00000008,
-    kIOAGP4xDataRateMode3	= 0x00000001,
-    kIOAGP8xDataRateMode3	= 0x00000002,
+    kGART_2xRESET		= 0x00010000
 };
 
 class IORangeAllocator;
@@ -181,7 +120,7 @@ class IORangeAllocator;
 
 class AppleMacRiscAGP : public AppleMacRiscPCI
 {
-    OSDeclareDefaultStructors(AppleMacRiscAGP);
+    OSDeclareDefaultStructors(AppleMacRiscAGP)
 
 protected:
     IORangeAllocator *	agpRange;
@@ -190,24 +129,14 @@ protected:
     IOPhysicalLength	systemLength;
     volatile UInt32 *	gartArray;
     IOByteCount		gartLength;
-    UInt32		gartCtrl;
-    UInt32		agpCommandMask;
-    UInt32		agpCommandSet;
     UInt8		targetAGPRegisters;
-    UInt8		isU3;
-    UInt8		isU32;
-    IOBufferMemoryDescriptor * dummyPage;
-    addr64_t		dummyPhys;
-    OSData *		gartHandle;	// Handle returned by IOMapper
 
 private:
     virtual IOReturn setAGPEnable( IOAGPDevice * master, bool enable,
 					IOOptionBits options = 0 );
 
-    inline void configSetClearMask( IOPCIAddressSpace space,
-					  UInt8 offset, UInt32 data, UInt32 mask );
-
 public:
+
     virtual bool configure( IOService * provider );
 
     virtual IOPCIDevice * createNub( OSDictionary * from );
@@ -244,7 +173,6 @@ public:
 				       IOMemoryDescriptor * memory,
 				       IOByteCount agpOffset,
 				       IOOptionBits options = 0 );
-
 };
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
